/*
** pork_format.c - helper functions for formatting text.
** Copyright (C) 2002-2003 Amber Adams <amber@ojnk.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#include <config.h>

#include <unistd.h>
#include <ncurses.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <sys/types.h>

#include <libfaim/aim.h>
#include <libfaim/faimconfig.h>

#include <pork.h>
#include <pork_missing.h>
#include <pork_util.h>
#include <pork_list.h>
#include <pork_buddy.h>
#include <pork_set.h>
#include <pork_swindow.h>
#include <pork_imwindow.h>
#include <pork_slist.h>
#include <pork_buddy_list.h>
#include <pork_acct.h>
#include <pork_cstr.h>
#include <pork_misc.h>
#include <pork_input.h>
#include <pork_bind.h>
#include <pork_events.h>
#include <pork_screen.h>
#include <pork_screen_cmd.h>
#include <pork_format.h>
#include <pork_color.h>

extern struct screen screen;

void make_timestamp(const char *work_format, char *buf, size_t len) {
	u_int32_t i;
	u_int32_t s = 0;
	u_int32_t time_12hour = 0;
	time_t cur_time;
	struct tm *tm;
	char tmp[CHAR_SHORTBUF];

	cur_time = time(NULL);
	tm = localtime(&cur_time);

	if (tm == NULL) {
		*buf = '\0';
		return;
	}

	len--;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			*tmp = '\0';
			switch (work_format[++i]) {
				case 'H':
					snprintf(tmp, sizeof(tmp), "%02d", tm->tm_hour);
					break;

				case 'h':
					if (tm->tm_hour == 0)
						time_12hour = 12;
					else {
						time_12hour = tm->tm_hour;

						if (time_12hour > 12)
							time_12hour -= 12;
					}

					snprintf(tmp, sizeof(tmp), "%02d", time_12hour);
					break;

				case 'm':
					snprintf(tmp, sizeof(tmp), "%02d", tm->tm_min);
					break;

				case 's':
					snprintf(tmp, sizeof(tmp), "%02d", tm->tm_sec);
					break;

				case 'z':
					snprintf(tmp, sizeof(tmp), "%s",
						(tm->tm_hour >= 12) ? "pm" : "am");
					break;
			}

			if (*tmp != '\0') {
				u_int32_t j = 0;

				while (tmp[j] != '\0' && s < len)
					buf[s++] = tmp[j++];
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}

void make_activity(const char *work_format, char *buf, size_t len) {
	u_int32_t i;
	u_int32_t s = 0;
	dlist_t *cur = screen.window_list;
	u_int32_t activity = 0;
	char digit[CHAR_SHORTBUF];
	char tmp[CHAR_SHORTBUF];

	*tmp = '\0';
	len--;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			if (work_format[++i] == 'A' && !activity) {
				do {
					struct imwindow *imwindow = cur->data;

					if (imwindow->swindow.activity &&
						!imwindow->swindow.visible)
					{
						if (activity > 0) {
							snprintf(digit, sizeof(digit),
								",%u", imwindow->refnum);
						} else {
							snprintf(digit, sizeof(digit),
								"%u", imwindow->refnum);
						}

						xstrncat(tmp, digit, sizeof(digit));
						activity++;
					}

					cur = cur->next;

				} while (cur != screen.window_list);

				if (activity) {
					u_int32_t j = 0;

					while (tmp[j] != '\0' && s < len)
						buf[s++] = tmp[j++];
				} else {
					*buf = '\0';
					return;
				}
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}

void make_typing(	const char *work_format,
					char *buf,
					size_t len,
					struct imwindow *imwindow)
{
	u_int32_t i;
	u_int32_t s = 0;

	len--;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			if (work_format[++i] == 'Y') {
				char *tmp;
				u_int32_t j = 0;

				tmp = NULL;

				if (imwindow->typing) {
					if (imwindow->typing == 1)
						tmp = opt_get_str(OPT_TEXT_TYPING_PAUSED);
					else
						tmp = opt_get_str(OPT_TEXT_TYPING);

					while (tmp[j] != '\0' && s < len)
						buf[s++] = tmp[j++];
				} else {
					*buf = '\0';
					return;
				}
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}

void make_held(	const char *work_format,
				char *buf,
				size_t len,
				struct imwindow *imwindow)
{
	char tmp[CHAR_SHORTBUF];
	u_int32_t i;
	u_int32_t j;
	u_int32_t s = 0;

	len--;

	*buf = '\0';

	if (imwindow->swindow.held == 0)
		return;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			if (work_format[++i] == 'H') {
				snprintf(tmp, sizeof(tmp), "%02d", imwindow->swindow.held);

				for (j = 0 ; tmp[j] != '\0' && s < len ; j++)
					buf[s++] = tmp[j];

			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}
	buf[s] = '\0';
}

void make_idle(	const char *work_format,
				char *buf,
				size_t len,
				struct aim_priv *priv,
				int hide_if_zero)
{
	char tmp[CHAR_SHORTBUF];
	u_int32_t i;
	u_int32_t j;
	u_int32_t s = 0;

	len--;

	*buf = '\0';

	if (priv->idle_time == 0 && hide_if_zero)
		return;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			if (work_format[++i] == 'I') {
				time_to_str(priv->idle_time, tmp, sizeof(tmp));

				for (j = 0 ; tmp[j] != '\0' && s < len ; j++)
					buf[s++] = tmp[j];

			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}
	buf[s] = '\0';
}

void make_warn(	const char *work_format,
				char *buf,
				size_t len,
				struct aim_priv *priv,
				int hide_if_zero)
{
	char tmp[CHAR_SHORTBUF];
	u_int32_t i;
	u_int32_t j;
	u_int32_t s = 0;

	len--;

	*buf = '\0';

	if (priv->warn_level == 0 && hide_if_zero)
		return;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			if (work_format[++i] == 'W') {
				snprintf(tmp, sizeof(tmp), "%02d", priv->warn_level);

				for (j = 0 ; tmp[j] != '\0' && s < len ; j++)
					buf[s++] = tmp[j];

			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}
	buf[s] = '\0';
}


void make_im(	const char *work_format,
				char *buf,
				size_t len,
				const char *nick,
				const char *msg)
{
	u_int32_t i;
	u_int32_t s = 0;
	const char *tmp;

	len--;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			switch (work_format[++i]) {
				case 'n':
				case 'N':
					tmp = nick;
					break;

				case 'm':
				case 'M':
					tmp = msg;
					break;

				default:
					tmp = NULL;
					break;
			}

			if (tmp != NULL) {
				u_int32_t j = 0;

				/*
				** We need to check for '%' characters in
				** the message so we don't fuck up the display
				** if someone sends us a message containing one.
				**
				** If an '%' is encountered, it's doubled, so when it's
				** converted to a cstr, it ends up being displayed
				** as a single '%'.
				*/

				while (tmp[j] != '\0' && s < len) {
					if (tmp[j] == '%') {
						buf[s++] = '%';

						if (s >= len)
							break;
					}

					buf[s++] = tmp[j++];
				}
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}

void make_chat_info(	const char *work_format,
						char *buf,
						size_t len,
						const char *name,
						const char *action,
						const char *room,
						int exchange)
{
	u_int32_t i;
	u_int32_t s = 0;
	char exchnum[4];
	const char *tmp;

	len--;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			switch (work_format[++i]) {
				case 'n':
				case 'N':
					tmp = name;
					break;

				case 'a':
				case 'A':
					tmp = action;
					break;

				case 'r':
				case 'R':
					tmp = room;
					break;

				case 'e':
				case 'E':
					snprintf(exchnum, sizeof(exchnum), "%02d", exchange);
					tmp = exchnum;
					break;

				default:
					tmp = NULL;
					break;
			}

			if (tmp != NULL) {
				u_int32_t j = 0;

				while (tmp[j] != '\0' && s < len)
					buf[s++] = tmp[j++];
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}

void make_chat_arginfo(	const char *work_format,
						char *buf,
						size_t len,
						const char *name,
						const char *room,
						int exchange,
						const char *altname,
						const char *msg)
{
	u_int32_t i;
	u_int32_t s = 0;
	char exchnum[4];
	const char *tmp;
	const char *blank = " ";

	len--;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			switch (work_format[++i]) {
				case 'n':
				case 'N':
					tmp = name;
					break;

				case 'u':
				case 'U':
					tmp = altname;
					break;

				case 'm':
				case 'M':
					if (msg == NULL)
						tmp = blank;
					else
						tmp = msg;
					break;

				case 'r':
				case 'R':
					tmp = room;
					break;

				case 'e':
				case 'E':
					snprintf(exchnum, sizeof(exchnum), "%02d", exchange);
					tmp = exchnum;
					break;

				default:
					tmp = NULL;
					break;
			}

			if (tmp != NULL) {
				u_int32_t j = 0;

				/*
				** Fix anything that could be mistaken
				** for a color code by doubling any % chars.
				*/
				if (tolower(work_format[i]) == 'm') {
					while (tmp[j] != '\0' && s < len) {
						if (tmp[j] == '%') {
							buf[s++] = '%';

							if (s >= len)
								break;
						}

						buf[s++] = tmp[j++];
					}
				} else {
					while (tmp[j] != '\0' && s < len)
						buf[s++] = tmp[j++];
				}
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}

void make_warning(	const char *work_format,
					char *buf,
					size_t len,
					const char *you,
					const char *warner,
					u_int16_t warn_level)
{
	const char *tmp;
	char warn[CHAR_SHORTBUF];
	u_int32_t i;
	u_int32_t s = 0;

	len--;

	*buf = '\0';

	if (work_format == NULL)
		return;

	for (i = 0 ; work_format[i] != '\0' && s < len ; i++) {
		if (work_format[i] == FORMAT_VARIABLE) {
			switch (work_format[++i]) {
				case 'n':
				case 'N':
					tmp = you;
					break;

				case 'u':
				case 'U':
					tmp = warner;
					break;

				case 'w':
				case 'W':
					snprintf(warn, sizeof(warn), "%u", warn_level);
					tmp = warn;
					break;

				default:
					tmp = NULL;
					break;
			}
			if (tmp != NULL) {
				u_int32_t j = 0;

				while (tmp[j] != '\0' && s < len)
					buf[s++] = tmp[j++];
			} else {
				buf[s++] = work_format[--i];
			}
		} else {
			buf[s++] = work_format[i];
		}
	}

	buf[s] = '\0';
}
